//////////////////Thermal.c ////// for PIC16F684 using MPLAB IDE 7.51
//      PICC Compiler with PICSTART Plus programmer ///////
//      by Dick Aidt  7 Mar 07  //
#include <pic.h>
#include <math.h>
__CONFIG(INTIO & WDTDIS & PWRTEN & MCLRDIS & UNPROTECT &\
UNPROTECT & BORDIS & IESODIS & FCMDIS);
int A [ ]= {0,1,0,1};
int B [ ]= {0,0,1,1};
float Temps [ 4 ];
float Value;
int long t;
int d, m, n, s, x, y;
/////////  To adjust the sensors to all read the same  /////////
float  Adjmt [ ] = {0.0, 0.0, 0.0, 0.0 };    /* for adjusting each sensor temp.
	Each sensor is adjusted to be the average of all four,
	where all sensors report the same temperature in F .
		0 = Top, 1 = Ctr, 2 = Bot, and 3 = Tbot			*/
///////////////////////////////////////////////////////////////////////////////////				
int ADC [ ] = { 0b11010001,	// Top = AN4
	            0b11000001,	// Ctr   = AN0
	            0b11010101,	// Bot  = AN5
	            0b11001101};	// Tbot= AN3 
float Degrees (float V);
void Send (float Value);
void Avalue (int y );
void Pause_ms (float z);	
main()
{	CMCON0 = 7;
	ANSEL = 0b00111011;
	ADCON1 = 0b00010000;
	TRISA = 0b00010011;	// AN0,3 and RA1 are inputs
	TRISC = 0b00000011;	// AN4,5 are inputs
	PORTA = 0;
	PORTC = 0;	
TOP:	for ( x = 0; x < 4; x ++)
	{	Avalue (x); // Get value into ADRESH & ADRESL
		Value = Degrees(Value);	// Turn data into degrees F
		Value = Value + Adjmt [ x ]; //Adjust the sensors 
		Temps [ x ] = Value;		
		RC4=A[ x ];	// Channel A, // send which led to light up
		RC5=B [ x ];	// Channel B,     
		Send (Value);		// Send value to be displayed
		NOP();	
		for (t = 0; t < 62000; t++);	// Display time= 1.5 sec
		NOP();
		Value = 0;	
	}

////////////////////pump on/off decision using Temps [ x ]//////////				
				// Top = Temps [ 0 ]
				// Ctr   = Temps [ 1 ]
				// Bot  = Temps [ 2 ]
				// Tbot= Temps [ 3 ]
	if (! RC2 &&  Temps [ 0 ] > ( Temps [ 3 ] + 5.0 )) // Top > Tank
		{ RC2 = 1;}
	if ( Temps [ 1 ]  < 37.0) // freeze protect
		{
		RC2 = 1;	// pump on
		RC3 = 1;	// freeze led on
		}		
	if ( RC2  &&  ! RC3 &&Temps [ 0 ] < (Temps [ 3 ] + 2.0 ))// Top < Tank
		{RC2 = 0;}
	if ( RC3 && (Temps [ 1 ] > 42.0)) // freeze protect done
		{
		RC2 = 0;	// pump off
		RC3 = 0;	// freeze led off
		}
	goto TOP;
} // End
///////////////////////////////////////////// Define Functions//////////////
float Degrees ( float V)	
{
	V=((ADRESH<<8) + ADRESL) * .234598; /* This multiply
	by number is an adjustment for the ADC reference 
	voltage. Right now the voltage is 2.5 V and is variable
	by the tolerance on the two 10 K resistors dividing the
	5 V power source. The group of sensors can be adjusted
	to the correct temperature using this adjustment. There
	is a 10 bit ADC so each bit is worth 2.3741 mv per degree C
	and the sensors are 10 mv per C  */

	V=(( V * 1.8) + 32.0); //convert to farenheit
	ADRESH = 0;
	ADRESL = 0;					
	return V;					
}	
////////////////////////////////////////////////////////////
void Send (float Value)
{//send data word.....i.e. 77.3 F, comes in on "Value"	
	m=1;  // bit mask
	n = Value * 10.0; // move decimal point
	RA2 = 0;               // start the receiver on the other end
		Pause_ms (30.0);// give rx a chance to see this	
	for (d = 0; d < 12; d++)	// setup for 12 bits to send
		{
		s = n & m;// get a bit
		RA5 = s;   // send the bit
			Pause_ms (0.25);
		RA2 = 1;  // sync HI
			Pause_ms (0.25);	
		RA2 = 0;  // sync LO
			Pause_ms (0.25);
		RA5 = 0;    // reset data pluse
		n = n >> 1; // shift to next bit
			Pause_ms (0.25);
		} // end for loop after 12 iterations
	 RA2 = 1;	// stop the receiver
}  //done sending 12 bits
/////////////////////////////////////////////////////////
void Avalue ( y )
{
	ADCON0 = ADC [ y ];
	GODONE = 1;
		do
		NOP();
		while (GODONE);
}
////////////////////////////////////////////////////////
void  Pause_ms (float z)
{
	z = z * 10.2;
	NOP();
	for (y = 0; y < z; y ++);
	NOP();
}
//////////////////////////////////////////////////////////////
